package com.zgiot.zx.workflow.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.generator.config.IFileCreate;
import com.zgiot.zx.workflow.entity.*;
import com.zgiot.zx.workflow.mapper.*;
import lombok.extern.slf4j.Slf4j;
import org.activiti.bpmn.converter.BpmnXMLConverter;
import org.activiti.bpmn.model.Process;
import org.activiti.bpmn.model.*;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.repository.Deployment;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;


import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;

//activiti动态读取xml文件
@Slf4j
@Component
public class ReadBpmn {

    @Value("${template.processfile.path}")
    private String processPath;

    private static final String REJECT = "驳回";
    private static final String PASS = "通过";
    private static final String ASSIGNEE = "#{START_USER}";
    @Autowired
    private WorkApproveMapper workApproveMapper;
    @Autowired
    private SUserMapper sUserMapper;
    @Autowired
    private SRoleMapper sRoleMapper;
    @Autowired
    private ProcessFileMapper processFileMapper;
    @Autowired
    RepositoryService repositoryService;
    @Autowired
    private ProcessGatewayMapper processGatewayMapper;

    public List<String> newProcessFile(String filePath, String templateId) throws IOException, XMLStreamException {
        String fileNamePath = processPath;
        InputStream inputStream = new FileInputStream(fileNamePath + filePath);

        //通过模板id查询approve表信息；
        QueryWrapper<TemplateApproveDO> qw = new QueryWrapper<>();
        qw.eq("template_id", templateId);
        //根据模板id查询网关表信息；
        QueryWrapper<TemplateGatewayDO> qw1 = new QueryWrapper<>();
        qw1.eq("template_id",templateId);
        List<TemplateGatewayDO> templateGatewayDOS = processGatewayMapper.selectList(qw1);
        List<TemplateApproveDO> list = workApproveMapper.selectList(qw);
        //设置角色
        List<Map<String, List<String>>> assignees = new ArrayList<>();
        Map<String, List<String>> assigneeMap = new HashMap<>();
        List<String> names = null;
        //设置角色
        List<Map<String, List<String>>> candidateGroups = new ArrayList<>();
        Map<String, List<String>> candidateMap = new HashMap<>();
        for (TemplateApproveDO templateApproveDO : list) {
            names = new ArrayList<>();
            if ("1".equals(templateApproveDO.getPersonType())) {
                String personData = templateApproveDO.getPersonData();
                String[] split = personData.split(",");
                for (int i = 0; i < split.length; i++) {
                    SUser sUser = sUserMapper.selectById(split[i]);
                    if (sUser!=null) {
                        names.add(sUser.getName());
                    }
                }
                assigneeMap.put(templateApproveDO.getActivityId(), names);
                assignees.add(assigneeMap);
            } else{
                String personData = templateApproveDO.getPersonData();
                String[] split = personData.split(",");
                for (int i = 0; i < split.length; i++) {
                    SRole sRole = sRoleMapper.selectById(split[i]);
                   if (sRole!=null) {
                       log.debug("---------------" + sRole.getCode());
                       names.add(sRole.getCode());
                   }
                }
                candidateMap.put(templateApproveDO.getActivityId(), names);
                candidateGroups.add(candidateMap);
            }
        }
       /* List<Map<String, List<String>>> candidateGroups = new ArrayList<>();
        Map<String, List<String>> candidateMap = new HashMap<>();
        List<String> tasklist1 = new ArrayList<>();
        tasklist1.add("业务部门一");
        tasklist1.add("业务部门三");
        candidateMap.put("userTask1", tasklist1);
        List<String> tasklist2 = new ArrayList<>();
        tasklist2.add("业务部门二");
        tasklist2.add("业务部门三");
        candidateMap.put("userTask2", tasklist2);
        candidateGroups.add(candidateMap);
        //设置活动id和与之对应的角色
        List<Map<String, List<String>>> assignees = new ArrayList<>();
        Map<String, List<String>> assigneeMap = new HashMap<>();
        List<String> names = new ArrayList<>();
        names.add("张三");
        names.add("李四");
        assigneeMap.put("userTask1", names);
        List<String> names1 = new ArrayList<>();
        names1.add("王二");
        names1.add("麻子");
        assigneeMap.put("userTask2", names1);
        assignees.add(assigneeMap);*/
        //创建bpmnxml解析对象
        BpmnXMLConverter converter = new BpmnXMLConverter();

        XMLInputFactory factory = XMLInputFactory.newInstance();
        BpmnModel bpmnModel = converter.convertToBpmnModel(factory.createXMLStreamReader(inputStream));
        Process process = bpmnModel.getMainProcess();

        //取出所有元素
        Collection<FlowElement> userTaskList = process.getFlowElements();
        //把经过的任务节点保存到list中
        List<UserTask> userTasks = new ArrayList<>();
        //遍历查询其中为userTask类型的元素
        process.findFlowElementsOfType(UserTask.class).forEach(userTask -> {
            userTasks.add(userTask);
            List<SequenceFlow> sequenceFlows = userTask.getOutgoingFlows();
            //2.设置线的判断条件
            //获取为userTask类型的节点，并获取其向外发出去的线
                //任务连接大于2条，则说明其有驳回和通过处理
            for (SequenceFlow flow : sequenceFlows) {
                if (StringUtils.isNotEmpty(flow.getName())) {
                    flow.setConditionExpression("#{message=='" + flow.getName() + "'}");
                }
                /*if (flow.getName().equals(REJECT)) {
                    //如果此节点指向第一个任务节点，则为驳回(3)
                    flow.setConditionExpression("#{message==\"3\"}");
                } else if (flow.getName().equals(PASS)) {
                    //反之设为通过
                    flow.setConditionExpression("#{message==\"2\"}");
                }*/
            }

        });
        //获取所有排他网关
        process.findFlowElementsOfType(Gateway.class).forEach(gateway -> {
            List<SequenceFlow> outgoingFlows = gateway.getOutgoingFlows();
            //设置线的判断条件
            for (SequenceFlow sequenceFlow: outgoingFlows) {
                //遍历模板对应的全部网关
                for (TemplateGatewayDO templateGatewayDO: templateGatewayDOS) {
                    if (templateGatewayDO.getActivityId().equals(sequenceFlow.getId())){
                        String Condition = templateGatewayDO.getConditions().substring(8);
                        sequenceFlow.setConditionExpression(Condition);
                        break;
                    }
                }
            }
        });

        //对所有活动节点进行遍历，找到节点类型为endEvent的
        FlowElement startElement = null;
        for (FlowElement flowElement : userTaskList) {
            //添加监听
            if (flowElement.getClass().equals(EndEvent.class)&&flowElement.getClass()!=null) {
                //如果元素名称为 endEvent就添加监听
                List<ActivitiListener> listeners = new ArrayList<>();
                ActivitiListener activitiListener = new ActivitiListener();
                activitiListener.setEvent("end");
                activitiListener.setImplementationType("class");
                activitiListener.setImplementation("org.rcisoft.common.ProcessStatusListener");
                listeners.add(activitiListener);
                flowElement.setExecutionListeners(listeners);
            } else if (flowElement.getClass().equals(StartEvent.class)) {
                startElement = flowElement;
            }
        }
        List<String> activitIds = new ArrayList<>();
        String activitId1 = "";
        for (UserTask userTask : userTasks) {
            if (userTask.getIncomingFlows().size()>0){
                activitId1 = userTask.getId();
                activitIds.add(activitId1);
            }
        }
        String activitId = "";
        for (UserTask userTask : userTasks) {
            if (userTask.getIncomingFlows().size()>0&&userTask.getIncomingFlows().get(0).getSourceRef().equals(startElement.getId())){
                activitId = userTask.getId();
            }
        }

        for (UserTask userTask : userTasks) {
            if (startElement.getId().equals(userTask.getIncomingFlows().get(0).getSourceRef())) {
                userTask.setAssignee(ASSIGNEE);
            } else {
                if (candidateGroups!=null){
                    for (Map<String, List<String>> candidateGroup : candidateGroups) {
                        if (candidateGroup.containsKey(userTask.getId())) {
                            //设置角色
                            userTask.setCandidateGroups(candidateGroup.get(userTask.getId()));
                        }
                    }
                }
                if (assignees!=null){
                    for (Map<String, List<String>> assignee : assignees) {
                        if (assignee.containsKey(userTask.getId())) {
                            //设置组任务，任务办理人
                            userTask.setCandidateUsers(assignee.get(userTask.getId()));
                        }
                    }
                }

            }
        }

       /*     if (userTask.getOutgoingFlows().size()<2) {
                System.out.println("首节点："+userTasks.get(0));
                userTasks.get(0).setAssignee(ASSIGNEE);
            } else if (userTask.getOutgoingFlows().size()>1){
                //1.为其设置任务角色和办理人
                System.out.println("UserTask类型ID：" + userTask.getId());*/


        // 这行代码是重新布局
        // new BpmnAutoLayout(bpmnModel).execute();
        String filename = String.valueOf(UUID.randomUUID());
        String filepath = filename + ".bpmn";
        processFileMapper.updateNewProcessFile(filepath,templateId,activitId);
        Files.write(Paths.get(processPath + filepath),
                converter.convertToXML(bpmnModel));
        FileInputStream fileInputStream = new FileInputStream(processPath+filepath);
        Deployment deploy = repositoryService.createDeployment().name(filename).addInputStream(filepath, fileInputStream).deploy();
        return activitIds ;
    }

}
