package com.zgiot.zx.workflow.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zgiot.zx.workflow.dto.WorkPermissionInfoResDTO;
import com.zgiot.zx.workflow.dto.WorkPermissionResDTO;
import com.zgiot.zx.workflow.entity.TemplatePermissionDO;

import com.zgiot.zx.workflow.mapper.WorkPermissionMapper;
import com.zgiot.zx.workflow.mapper.WorkTemplateFieldMapper;
import com.zgiot.zx.workflow.service.IWorkPermissionService;
import lombok.extern.slf4j.Slf4j;
import org.activiti.bpmn.converter.BpmnXMLConverter;
import org.activiti.bpmn.model.BpmnModel;
import org.activiti.bpmn.model.FlowElement;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-05-08
 */
@Service
@Slf4j
public class WorkPermissionServiceImpl extends ServiceImpl<WorkPermissionMapper, TemplatePermissionDO> implements IWorkPermissionService {

    Logger logger = LoggerFactory.getLogger(WorkPermissionServiceImpl.class);

    /*@Autowired
    ProcessFileMapper processFileMapper;*/

  /*  @Autowired
    WorkTemplateFieldMapper workTemplateFieldMapper;*/

    /*@Override
    public boolean saveWorkPermission(String jsonStr) {
        try {
            JSONObject json = JSON.parseObject(jsonStr);
            String templateId = json.getString("templateId");
            String activityId = json.getString("activityId");
            if (activityId == null || "".equals(activityId)) {
                return false;
            }
            //先通过流程节点ID清除掉之前的表单配置
            QueryWrapper qw = new QueryWrapper();
            qw.eq("ACTIVITY_ID", activityId);
            baseMapper.delete(qw);
            JSONArray template = json.getJSONArray("template");
            List<TemplatePermissionDO> templatePermissionDOList = new ArrayList<>();
            for (Object obj : template) {
                JSONObject fieldJson = (JSONObject) obj;
                TemplatePermissionDO templatePermissionDO = TemplatePermissionDO.builder().id(UUID.randomUUID().toString().replace("-", ""))
                        .templateId(templateId)
                        .activityId(activityId)
                        .permission(fieldJson.getString("permission"))
                        .fieldId(fieldJson.getString("fieldId")).build();
                templatePermissionDOList.add(templatePermissionDO);
            }
            this.saveBatch(templatePermissionDOList);

            return true;
        } catch (Exception e) {
            logger.error(e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return false;
    }*/

    @Override
    public boolean saveWorkPermissionBatch(String jsonStr) {
        try {
            JSONArray jsonArray = JSON.parseArray(jsonStr);
            jsonArray.size();
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject json = JSON.parseObject(jsonArray.get(i).toString());
                String templateId = json.getString("templateId");
                String activityId = json.getString("activityId");
                if (activityId == null || "".equals(activityId)) {
                    return false;
                }
                if (templateId != null && !"".equals(templateId)) {
                    //先通过流程节点ID清除掉之前的表单配置
                    QueryWrapper qw = new QueryWrapper();
                    qw.eq("activity_id", activityId);
                    qw.eq("template_id", templateId);
                    baseMapper.delete(qw);
                    JSONArray template = json.getJSONArray("form");
                    List<TemplatePermissionDO> templatePermissionDOList = new ArrayList<>();
                    for (Object obj : template) {
                        JSONObject fieldJson = (JSONObject) obj;
                        TemplatePermissionDO templatePermissionDO = TemplatePermissionDO.builder().id(UUID.randomUUID().toString().replace("-", ""))
                                .templateId(templateId)
                                .activityId(activityId)
                                .permission(fieldJson.getString("permission"))
                                .fieldId(fieldJson.getString("fieldId"))
                                .ifWrite(fieldJson.getString("ifWrite")).build();
                        templatePermissionDOList.add(templatePermissionDO);
                    }
                    this.saveBatch(templatePermissionDOList);
                }
            }
            return true;
        } catch (Exception e) {
            logger.error(e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return false;

    }

    @Override
    public List<WorkPermissionInfoResDTO> getWorkPermissionByTemplateId(String templateId) {
        List<WorkPermissionResDTO> WorkPermissionResDTOList =  baseMapper.selectListByTemplateId(templateId);
        Map<String,String> map = new HashMap<>();
        for (WorkPermissionResDTO workPermissionResDTO :WorkPermissionResDTOList){
            map.put(workPermissionResDTO.getActivityId(),"1");
        }


        List<WorkPermissionInfoResDTO> list =  new ArrayList<>();
        for (Map.Entry<String, String> entry : map.entrySet()) {
            WorkPermissionInfoResDTO workPermissionInfoResDTO = new WorkPermissionInfoResDTO();
            List<WorkPermissionResDTO> templatePermission = new ArrayList<>();
            for (WorkPermissionResDTO wp : WorkPermissionResDTOList) {
                if (entry.getKey().equals(wp.getActivityId())) {
                    templatePermission.add(wp);
                }
            }
            workPermissionInfoResDTO.setTemplateId(templateId);
            workPermissionInfoResDTO.setActivityId(entry.getKey());
            workPermissionInfoResDTO.setForm(templatePermission);
            list.add(workPermissionInfoResDTO);
        }

        return list;
    }

 /*   @Override
    public List<WorkPermissionResDTO> queryPermissionAndField(String templateId) {
        //通过节点id查询查询表单信息
        return baseMapper.queryPermissionAndFieldByNode("StartEvent_1", templateId);

    }*/

    /**
     * 根据流程文件获取流程文件的唯一ID
     *
     * @param bpmnPath
     * @return
     */
    public String queryBPMNkey(String bpmnPath) {
        File processFile = new File(bpmnPath);
        XMLInputFactory xif = XMLInputFactory.newInstance();
        InputStreamReader in;
        XMLStreamReader xtr;
        String activitiNodeId = "";
        try {
            in = new InputStreamReader(new FileInputStream(processFile));
            xtr = xif.createXMLStreamReader(in);
            BpmnModel model = new BpmnXMLConverter().convertToBpmnModel(xtr);
            if (model != null) {
                List<FlowElement> flowElements = (List) model.getMainProcess().getFlowElements();
                activitiNodeId = flowElements.get(0).getId();
            }
        } catch (Exception e) {
            logger.error("文件读取失败");
        }
        return activitiNodeId;
    }
}
