package com.zgiot.zx.workflow.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zgiot.zx.workflow.entity.TemplateFormFieldDO;
import com.zgiot.zx.workflow.mapper.WorkTemplateFieldMapper;
import com.zgiot.zx.workflow.service.IWorkTemplateFieldService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-05-08
 */
@Service
public class WorkTemplateFieldServiceImpl extends ServiceImpl<WorkTemplateFieldMapper, TemplateFormFieldDO> implements IWorkTemplateFieldService {

    Logger logger = LoggerFactory.getLogger(WorkTemplateFieldServiceImpl.class);

    @Override
    @Transactional
    public boolean saveOrUpdateWorkTempldateField(String templateId, String jsonStr) {
        try {
            JSONObject json = JSON.parseObject(jsonStr);
            //保存前先清除之前的信息
            QueryWrapper qw = new QueryWrapper();
            qw.eq("TEMPLATE_ID", templateId);
            baseMapper.delete(qw);
            JSONArray array = json.getJSONArray("template");
            List<TemplateFormFieldDO> insertList = new ArrayList<>();
            if (array != null && !array.isEmpty()) {
                for (Object obj : array) {
                    JSONObject levelFirst = (JSONObject) obj;
//                    JSONObject levelecond = levelFirst.getJSONObject("obj");
                    TemplateFormFieldDO wtf = TemplateFormFieldDO.builder().id(levelFirst.getString("id"))
                            .templateId(templateId)
                            .fieldNm(levelFirst.get("fieldNm").toString())
                            .fieldOrder(Integer.parseInt(levelFirst.get("fieldOrder").toString()))
                            .fieldTp(levelFirst.get("fieldTp").toString())
                            .major(levelFirst.get("major").toString())
                            .print(levelFirst.get("print").toString())
                            .required(levelFirst.get("required").toString())
                            .expand(levelFirst.get("expand").toString())
                            .build();
                    insertList.add(wtf);
                }
                this.saveBatch(insertList);
                return true;
            }

        } catch (Exception e) {
            logger.error(e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return false;
    }

    @Override
    public boolean deleteWorkTempldateField(String templateId) {
        try {
            QueryWrapper qw = new QueryWrapper();
            qw.eq("TEMPLATE_ID", templateId);
            baseMapper.delete(qw);
            return true;

        } catch (Exception e) {
            logger.error(e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return false;
    }
    @Override
    public List<TemplateFormFieldDO> selectWorkTempldateField(String templateId) {
        try {
            QueryWrapper qw = new QueryWrapper();
            qw.eq("TEMPLATE_ID", templateId);
            List<TemplateFormFieldDO> f = baseMapper.selectList(qw);
            return f;

        } catch (Exception e) {
            logger.error(e.getMessage());
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return null;
    }

   /* @Override
    public List<TemplateFormFieldDO> getWorkTempldateFieldList(String templateId) {
        QueryWrapper qw = new QueryWrapper();
        qw.eq("TEMPLATE_ID", templateId);
        qw.orderByAsc("FIELD_ORDER");
        return baseMapper.selectList(qw);
    }*/

}
